#!/usr/bin/perl
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.

#Vhffs::Note written by sod` <dieu AT gunnm DOT org>

# This software is free software, please read LICENCE file

use Vhffs::Constants;

use strict;
use diagnostics;

package Vhffs::Note;
sub set_note
{
	my $vhffs		=	shift;
	my $entity		=	shift;
	my $value		=	shift;

	my $oldvalue = get_note( $vhffs , $entity );
	my $newvalue = $value - $oldvalue;
	inc_note( $vhffs , $entity , $newvalue );
}



sub inc_note
{
	my $vhffs        	= shift;
	my $entity 			= shift;
	my $increase 		= shift;
	my $note   			= 0;
	my $already_exists 	= 0;

	return -1 if( $entity->fetch < 0 );


	#Fetch note if already exists
	my $query = "SELECT note FROM vhffs_notes WHERE object_id='".$entity->get_oid."'";
	my $request = $vhffs->{'db'}->{'DB_READ'}->prepare( $query );
	my $rows = $request->execute;
	if ( $rows > 0 )
	{
		$already_exists = 1;
		my $result = $vhffs->{'db'}->{'DB_READ'}->selectall_arrayref( $query );
		$note = $result->[0][0];
	}

	#Finally compute new note
	$note += $increase;


	if( $already_exists == 1 )
	{
		$query = "UPDATE vhffs_notes SET note='".$note."' WHERE object_id='".$entity->get_oid."'";
	}
	else
	{
		$query = "INSERT INTO vhffs_notes VALUES('".$entity->get_oid."' , '".$note."')";
	}
	$request = $vhffs->{'db'}->{'DB_WRITE'}->prepare( $query );
	
	$request->execute;

	return 1;
}

sub get_note
{
	my $vhffs			=	shift;
	my $entity 			= 	shift;
	
	my $query = "SELECT note FROM vhffs_notes WHERE object_id='".$entity->get_oid."'";

	my $request = $vhffs->{'db'}->{'DB_READ'}->prepare( $query );
	my $rows = $request->execute;

	if ( $rows > 0 )
	{
		my $result = $vhffs->{'db'}->{'DB_READ'}->selectall_arrayref( $query );
		return $result->[0][0];
	}
	else
	{
		return 0;
	}
	
}

1;

__END__

=head1 NAME

	Vhffs::Note - handle and manage note for objects on VHFFS platform

=head1 SYNOPSIS

	use Vhffs::Note;
	use Vhffs::Main;

	$vhffs = init Vhffs::Main;

	...
	(considers that you have a VHFFS object in $object)
	...

	print "Current note for your object : " . Vhffs::Note::get_note( $vhffs , $object );
	
	#We add 3 to the current note
	Vhffs::Note::inc_note( $vhffs , $object , 3 );

	#We set the note to 0
	Vhffs::Note::set_note( $vhffs , $object , 0 );

	#We decrease the note of 1
	Vhffs::Note::inc_note( $vhffs , $object , -1 );


=head1 METHODS

	get_note( $vhffs , $object ) : return the note for the given object

	set_note( $vhffs , $object , $value ) : set the note

	inc_note( $vhffs , $object , $inc ) : add $inc to the current not of object $object

=head1 AUTHOR

	Julien Delange <julien at gunnm dot org>

=head1 COPYRIGHT

	Copyright 2006 Julien Delange
