#!/usr/bin/perl
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


package Vhffs::Robots::Group;

use Vhffs::Group;
use Vhffs::User;
use Vhffs::Functions;
use Vhffs::Constants;

sub get_all_groups_to_create
{
	my( $main ) = @_;
	
	my $query = "SELECT g.groupname,g.gid FROM  vhffs_groups g, vhffs_object o WHERE o.state='".Vhffs::Constants::WAITING_FOR_CREATION."' AND g.object_id = o.object_id";

	$request = $main->{'db'}->{'DB_READ'}->prepare( $query ) or return -2;
	return undef if ( $request->execute() <= 0 );

	return( $request->fetchall_hashref('groupname') );
}

sub get_all_groups_refused
{
    my( $main ) = @_;

    my $query = "SELECT g.groupname,g.gid FROM  vhffs_groups g, vhffs_object o WHERE o.state='".Vhffs::Constants::VALIDATION_REFUSED."' AND g.object_id = o.object_id";

    $request = $main->{'db'}->{'DB_READ'}->prepare( $query ) or return -2;
    return undef if ( $request->execute() <= 0 );

    return( $request->fetchall_hashref('groupname') );
}



sub create_groupdir
{
	my $main = shift;
	my $group = shift;

	my $hash;
	if( ( defined $group ) && ( $group->fetch > 0 ) )
	{
		my $user = new Vhffs::User( $main , $group->get_owner_username , 401 );
		my $hash = Vhffs::Functions::hash_groupname( $group->get_groupname , $main );	
		$group->add_history("Ok, Robots will create the directory");
		Vhffs::Functions::create_dir( $hash );
		$group->add_history("Ok, Robots finish create the directory");
		if( defined $group->get_owneruid )
		{
			chown( $group->get_owneruid , $group->get_gid , $hash );
		}
		chmod( 0555 , $hash );
		add_user( $user , $group , $main );
		$group->set_status( Vhffs::Constants::ACTIVATED );
		$group->commit;
	}
	return $hash;
}


sub delete_groupdir
{
	my $main = shift;
	my $group = shift;
	my $cmd;


	my $hash;
	if( ( defined $group ) && ( $group->fetch > 0 ) )
	{
		my $user = new Vhffs::User( $main , $group->get_owner_username , 401 );
		my $hash = Vhffs::Functions::hash_groupname( $group->get_groupname , $main );	
		if( -d $hash )
		{
			$cmd = "rm -f $hash/* ";
			system( $cmd );

			$cmd = "rmdir  $hash ";
			system( $cmd );
		}
	}

	return 0;
}




sub getall_user_group_to_delete
{
	my $main = shift;	
	my $query = "SELECT * FROM vhffs_user_group WHERE state='".Vhffs::Constants::TO_DELETE."'";

	$request = $main->{'db'}->{'DB_READ'}->prepare( $query ) or return -2;
	return undef if ( $request->execute() <= 0 );

	return( $request->fetchall_hashref('gid') );
}


sub getall_user_group_to_create
{
	my $main = shift;	
	my $query = "SELECT * FROM vhffs_user_group WHERE state='".Vhffs::Constants::WAITING_FOR_CREATION."'";

	$request = $main->{'db'}->{'DB_READ'}->prepare( $query ) or return -2;
	return undef if ( $request->execute() <= 0 );

	return( $request->fetchall_hashref('gid') );
}

sub getall_user_group_to_delete
{
	my $main = shift;	
	my $query = "SELECT * FROM vhffs_user_group WHERE state='".Vhffs::Constants::TO_DELETE."'";

	$request = $main->{'db'}->{'DB_READ'}->prepare( $query ) or return -2;
	return undef if ( $request->execute() <= 0 );

	return( $request->fetchall_hashref('gid') );
}



sub del_user
{
	my $user = shift;
	my $group = shift;
	my $main = shift;

	return -1 if( $user->fetch < 0);
	return -2 if( $group->fetch < 0);

	my $path = $user->get_home . "/" . $group->get_groupname;
	unlink( $path );
	$group->add_history("The user ".$user->get_username." leave the group");
	$user->add_history("The user ".$user->get_username." leave the group");

	my $query = "DELETE FROM vhffs_user_group WHERE uid='".$user->get_uid."' AND gid='".$group->get_gid."'";
    my $request = $main->{'db'}->{'DB_WRITE'}->prepare( $query ) or return -2;
    $request->execute;

}


sub add_user
{
	my $user = shift;
	my $group = shift;
	my $main = shift;
	
	return -1 if( $user->fetch < 0);
	return -1 if( $group->fetch < 0);

	my $path = $user->get_home . "/" . $group->get_groupname;

	symlink( Vhffs::Functions::hash_groupname( $group->get_groupname , $main ) , $path );
	$group->add_history("The user ".$user->get_username." join the group");
	$user->add_history("The user ".$user->get_username." join the group");

	my $query = "UPDATE vhffs_user_group SET state='".Vhffs::Constants::ACTIVATED."' WHERE uid='".$user->get_uid."' AND gid='".$group->get_gid."'";
    my $request = $main->{'db'}->{'DB_WRITE'}->prepare( $query ) or return -2;
    $request->execute;
	
	return 1;
}

sub delete
{
	my $vhffs = shift;
	my $group = shift;
	my $users = $group->get_users;
	my $user;
	foreach $user ( @{$users} )
	{
		del_user( $user , $group , $vhffs );
	}

	delete_groupdir( $vhffs , $group );
	$group->delete;
}


1;

