#!/usr/bin/perl
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


package Vhffs::Robots::User;

use Vhffs::User;
use Vhffs::Constants;

sub getall_active_users
{
	my $main = shift;

	my $query = "SELECT u.username,u.uid FROM  vhffs_users u, vhffs_object o WHERE o.state='".Vhffs::Constants::ACTIVATED."' AND u.object_id = o.object_id";

	$request = $main->{'db'}->{'DB_READ'}->prepare( $query ) or return -2;
	return undef if ( $request->execute() <= 0 );

	return( $request->fetchall_hashref('username') );
}

sub get_all_home_to_create
{
	my( $main ) = @_;
	
	my $query = "SELECT u.username,u.uid,u.homedir FROM  vhffs_users u, vhffs_object o WHERE o.state='".Vhffs::Constants::WAITING_FOR_CREATION."' AND u.object_id = o.object_id";

	$request = $main->{'db'}->{'DB_READ'}->prepare( $query ) or return -2;
	return undef if ( $request->execute() <= 0 );

	return( $request->fetchall_hashref('username') );
	
}

sub create
{
	my $user = shift;
	return if( $user->fetch < 0);

	create_home( $user->{'main'} , $user->get_username );
}

sub delete
{
	my $vhffs = shift;
	my $user = shift;
	return if( $user->fetch < 0 );
	delete_home( $vhffs , $user );
}



sub delete_home
{
	my( $main , $user , $homedir , $cmd );

	$main = shift;
	$user = shift;

	if( $user->fetch > 0 )
	{
		$user->add_history( "Ok, robots takes now the destruction of this user" );
		$homedir = $user->get_home;

		if( $homedir && -d $homedir )
		{
			$cmd = "rm -rf $homedir";
			system( $cmd );
		}
	}

	return 1;
}


sub create_home
{
	my( $main , $username , $user , $homedir , @subdirs , $path , $value );
	$main = shift;
	$user = shift;

	if( $user->fetch > 0 )
	{
		$user->add_history( "Ok, robots take now the creation of the user" );
		$homedir = $user->get_home;

		@subdirs = split( /\// , $homedir );
		@subdirs = reverse @subdirs;
		
		$path = "";
		
		while( defined ( $value = pop ( @subdirs ) )  )
		{
			$path .= "/".$value if( $value ne '' );
			Vhffs::Functions::create_dir( $path );
			#return -1 if( mkdir( $path ) == false );
		}
		chown $user->get_uid , $user->get_gid , $path ;
		
		$user->add_history( "Creation complete for user. Homedir is now created." );
		$user->set_status( Vhffs::Constants::ACTIVATED );
		my $plainpassword = $user->generate_password;

		# Send a mail with plain password inside
		my $subject = "Welcome on VHFFS";
		my $content = sprintf("Hello %s %s, Welcome on VHFFS\n\nHere are your login information :\nUser: %s\nPassword: %s\n\nVHFFS Administrators\n", $user->get_firstname, $user->get_lastname, $user->get_username, $plainpassword );
		$user->send_mail_user( $subject, $content );

		if( $user->commit < 0 )
		{
			return -1;
		}
		else
		{
			return 1;
		}
	}

	return 1;

}


1;

