#!/usr/bin/perl
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


# This file is a part of VHFFS4 software, a hosting platform suite
# Please respect the licence of this file and whole program

# Author : soda < dieu at gunnm dot org >

package Vhffs::Services::Mail;

use base qw(Vhffs::Object);
use strict;
use DBI;


# Create a new instance of the current class
sub new
{
    my $main;
    my $domain;
    my $this;
    my $self;
    my $class;
	my $user;
	my $group;

    ( $class , $main , $domain , $user , $group ) = @_;
    
    $this = {};
    
    #If the domain is not specified by argument, return an undef instance
    return undef if ( ! defined $domain );
    
    #return undef if( $user->fetch < 0);
    $this = $class->SUPER::new( $main , 1 , '401' );
    
    return undef if( ! ( $domain =~ /^[a-zA-Z0-9\-\.]+$/)  );
    return undef if( length( $domain ) < 5 );
    
    $this->{'domain'} = $domain if ( defined $domain );
    
    # $this->{'owner_uid'} = $user->{'uid'} if ( defined $user->{'uid'} );
    $this->{'user'} = $user;
    $this->{'group'} = $group;

    $this->{'boxes'} = {};
    
    bless( $this , $class );
    
    return $this;
}

sub delete
{
    my $self = shift;
	my $query;
	my $request;


    $query = "DELETE FROM vhffs_mxdomain WHERE object_id='".$self->{'object_id'}."'";
    $request = $self->{'db'}->{'DB_WRITE'}->prepare($query);
    $request->execute or return -1;

    $query = "DELETE FROM vhffs_boxes WHERE domain='".$self->{'domain'}."'";
    $request = $self->{'db'}->{'DB_WRITE'}->prepare($query);
    $request->execute or return -1;

    $query = "DELETE FROM vhffs_forward WHERE domain='".$self->{'domain'}."'";
    $request = $self->{'db'}->{'DB_WRITE'}->prepare($query);
    $request->execute or return -1;





    return -2 if( $self->SUPER::delete < 0 );

    return 1;
}



# Create the object in the database
sub create
{
	my $self = shift;

	# If user or group^does not exists in the VHFFS database, we return -1, the object is not created
	return -1 if( $self->{'user'}->fetch < 0 );
	return -1 if( $self->{'group'}->fetch < 0 );

	$self->{'owner_uid'} = $self->{'user'}->get_uid();

	my $request = $self->{'db'}->{'DB_READ'}->prepare("SELECT * FROM vhffs_mxdomain WHERE domain='" . $self->{'domain'} . "'");
    my $rows = $request->execute;
    return -2 if ( $rows != 0 );

    my $oid = $self->SUPER::create();
    return -3 if ( $oid lt 0 );

	#Get the max mx_id and compute the new mx_id
    my $mx_id = 1;
    my $resultuid = $self->{'db'}->{'DB_READ'}->selectall_arrayref("SELECT MAX(mxdomain_id) FROM vhffs_mxdomain");
    $mx_id = $resultuid->[0][0] if ( defined $resultuid->[0][0] );
    $mx_id++;

    my $domainhash = Vhffs::Functions::hash_mxdomain( $self->{'domain'} );

	#Finally, create the object in the database
	my $query = "INSERT INTO vhffs_mxdomain VALUES( '".$mx_id."' , '".$self->{'domain'} . "', 'exim' , '".$domainhash."' , '0' , '' , '".$self->{'user'}->get_uid."' , '".$self->{'group'}->get_gid."', '$oid'  )";
	$request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
	$request->execute or return -2;

	return $mx_id;
}


# Get all data about this mail object in the databse
# Informations about the mail domain is stored in $ref->
# Inforations about forwards are stored in $ref->{'forward'}
# Informations about mailboxes are stored in $ref->{'boxes'}
sub fetch
{
    my $self = shift;
    
    my ($request , $result , $name , $request2 , $result2 , $names);
    
    $request = $self->{'db'}->{'DB_READ'}->prepare("SELECT * FROM vhffs_mxdomain where domain='".$self->{'domain'}."'") or return -1;
    return -3 if( $request->execute() <= 0 );
    $result = $request->fetchrow_hashref();
    
    foreach ( keys %{$result} )
    {
	$self->{$_} = $result->{$_}
    }
    
    #Fetch the object part( SUPER class)
    return -2 if ($self->SUPER::fetch < 0 ); 
    
    ###########################
    #Get all Boxes
    ###########################
    #So, we will have :
    #                      - $ref->{'boxes'}{$local_part}{'domain'}
    #                      - $ref->{'boxes'}{$local_part}{'local_part'}
    #                      - $ref->{'boxes'}{$local_part}{'domain_hash'}
    #                      - $ref->{'boxes'}{$local_part}{'password_hash'}
    #                      - $ref->{'boxes'}{$local_part}{'mbox_name'}
    #                      - $ref->{'boxes'}{$local_part}{'password'}
    $request = $self->{'db'}->{'DB_READ'}->prepare("SELECT * FROM vhffs_boxes where domain='".$self->{'domain'}."'") or return -1;
    $request->execute() or return -3;
    $result = $request->fetchall_hashref('local_part');	

	$self->{'boxes'} = {};
	foreach $name ( keys %{$result} )
	{
		foreach( keys %{$result->{$name}} )
		{
			$self->{'boxes'}{$name}{$_} = $result->{$name}{$_};
		}
		$self->{'boxes'}{$name}{'password'} = $self->{'boxes'}{$name}{'password_hash'};
		$self->{'boxes'}{$name}{'changed'} = 0;
		$self->{'boxes'}{$name}{'nospam'} = 1 if( $self->{'boxes'}{$name}{'nospam'} eq 't' );
		$self->{'boxes'}{$name}{'novirus'} = 1 if( $self->{'boxes'}{$name}{'novirus'} eq 't' );

	}
	######### End of boxes fetching ########




	######################
	#Get All forwarders
	######################
	#So, we will have :
	#                      - $ref->{'forward'}{$local_part}{'domain'}
	#                      - $ref->{'forward'}{$local_part}{'local_part'}
	#                      - $ref->{'forward'}{$local_part}{'remote_name'}
	#                      - $ref->{'forward'}{$local_part}{'password'}
    $request = $self->{'db'}->{'DB_READ'}->prepare("SELECT * FROM vhffs_forward where domain='".$self->{'domain'}."'") or return -1;
    $request->execute() or return -3;
    $result = $request->fetchall_hashref('local_part');	

	$self->{'forward'} = {};
	foreach $name ( keys %{$result} )
	{
		foreach( keys %{$result->{$name}} )
		{
			$self->{'forward'}{$name}{$_} = $result->{$name}{$_};
		}
		$self->{'forward'}{$name}{'changed'} = 0;
	}
	######### End of forward fetching ######
	
	return 1;	
}


# Commit all changes of the current instance in the database
sub commit
{
	use Vhffs::Functions;

	my $self = shift;	

	my( $request , $result , $rows , $query , $name , $password);

	$self->{'catchall'} = "" if( ! defined $self->{'catchall'} );

	$query = "UPDATE vhffs_mxdomain SET catchall='".$self->{'catchall'}."' WHERE domain='".$self->{'domain'}."'";
	$request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
	$request->execute;

	#COMMIT THE BOX PART
	foreach $name ( keys %{$self->{'boxes'}} )
	{
		next if( ( defined $self->{'boxes'}{$name}{'changed'} ) && ( $self->{'boxes'}{$name}{'changed'} == 0 ) );
		
    	$request = $self->{'db'}->{'DB_READ'}->prepare("SELECT * FROM vhffs_boxes WHERE domain='" . $self->{'domain'} . "' AND local_part='".$name."'");
	    $rows = $request->execute;

		#If boxes does not have nospam or novirus attribute
		#so, we set t to 0
		# This will create the attribute and initialize it
		$self->{'boxes'}{$name}{'novirus'} = 0 if( ! defined( $self->{'boxes'}{$name}{'novirus'} ));
		$self->{'boxes'}{$name}{'nospam'} = 0 if( ! defined( $self->{'boxes'}{$name}{'nospam'} ));

		if( $rows == 0 ) #PopBox NOT created, we must create it
		{
			my $domainhash = Vhffs::Functions::hash_mxdomain( $self->{'domain'} );
			my $userhash = Vhffs::Functions::hash_popuser( $name );
			my $passwordhash = "";

			$query = "INSERT INTO vhffs_boxes(domain,local_part,domain_hash,password_hash,mbox_name,password,nospam,novirus) VALUES('". $self->{'domain'} ."' , '". $name ."' , '". $domainhash ."' , '".$self->{'boxes'}{$name}{'password'}."' , '".$userhash."' , '' ,'".$self->{'boxes'}{$name}{'nospam'}."' , '".$self->{'boxes'}{$name}{'novirus'}."')";
			$request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
			$request->execute;
		}
		else	#Popbox already created
		{

			$query = "UPDATE vhffs_boxes SET nospam='".$self->{'boxes'}{$name}{'nospam'}."', novirus='".$self->{'boxes'}{$name}{'novirus'}."', password_hash='".$self->{'boxes'}{$name}{'password'}."' WHERE local_part='".$self->{'boxes'}{$name}{'local_part'}."' AND domain='".$self->{'domain'}."'";
			$request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
			$request->execute;
		}
	}	


	#COMMIT THE FORWARD PART
	foreach $name ( keys %{$self->{'forward'}} )
	{
		next if( ( defined $self->{'forward'}{$name}{'changed'} ) && ( $self->{'forward'}{$name}{'changed'} == 0 ) );
#		next if( ! defined $self->{'forward'}{$name}{'remote'} );
    	$request = $self->{'db'}->{'DB_READ'}->prepare("SELECT * FROM vhffs_forward WHERE domain='" . $self->{'domain'} . "' AND local_part='".$name."'");
	    $rows = $request->execute;

		if( $rows == 0 ) #PopBox NOT created, we must create it
		{
			$password = "";
#			$query = "INSERT INTO vhffs_forward VALUES('". $self->{'domain'} ."' , '". $name ."' , '". $self->{'forward'}{$name}{'remote_name'} ."' , '".$self->{'forward'}{$name}{'password'}."')";
			$query = "INSERT INTO vhffs_forward VALUES('". $self->{'domain'} ."' , '". $name ."' , '". $self->{'forward'}{$name}{'remote_name'} ."' , '4pr1lsUx')";
			$request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
			$request->execute;
		}
		else	#Popbox already created
		{
			$query = "UPDATE vhffs_forward SET remote_name='".$self->{'forward'}{$name}{'remote_name'}."' WHERE local_part='".$self->{'forward'}{$name}{'local_part'}."' AND domain='".$self->{'domain'}."'";
			$request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
			$request->execute;
		}
	}	





	return -3 if( $self->SUPER::commit < 0 );

	return 1;
}

sub change_forward
{
	my $self = shift;
	my $local_part = shift;
	my $destination = shift;


	$self->{'forward'}{$local_part}{'remote_name'} = $destination;
	$self->{'forward'}{$local_part}{'changed'} = 1;

	return 1;
}

sub change_box_password
{
	my $self = shift;
	my $local_part = shift;
	my $password = shift;

	return -1 if( ! defined $self->{'boxes'}{$local_part} );
	return -1 if( ! ( $password =~ /^[a-zA-Z0-9\_\-]+$/ ) );

	$self->{'boxes'}{$local_part}{'password'} = crypt_pwd( $password );
	$self->{'boxes'}{$local_part}{'changed'} = 1;

	return 1;
}

# The change_spam_status function change the status of a box 
# on the mail domain given in parmeter
# If the spam removal was enable, it will be disable at next commit() on object
sub change_spam_status
{
	my $self = shift;
	my $local_part = shift;

	return -1 if( ! defined $self->{'boxes'}{$local_part} );

	if( ! defined( $self->{'boxes'}{$local_part}{'nospam'} ) )
	{
		$self->{'boxes'}{$local_part}{'nospam'} = 0;
	}

	$self->{'boxes'}{$local_part}{'nospam'} = ( ( $self->{'boxes'}{$local_part}{'nospam'} + 1 ) % 2 );
	$self->{'boxes'}{$local_part}{'changed'} = 1;
	return 1;
}


#work as change_spam_status
sub change_virus_status
{
	my $self = shift;
	my $local_part = shift;

	return -1 if( ! defined $self->{'boxes'}{$local_part} );

	if( ! defined( $self->{'boxes'}{$local_part}{'novirus'} ) )
	{
		$self->{'boxes'}{$local_part}{'novirus'} = 0;
	}

	$self->{'boxes'}{$local_part}{'novirus'} = ( ( $self->{'boxes'}{$local_part}{'novirus'} + 1 ) % 2 );
	$self->{'boxes'}{$local_part}{'changed'} = 1;
	return 1;
}



sub addforward
{
	my $self = shift;
	my $name = shift;
	my $remote = shift;

	return -1 if( ( ! defined $name ) || ( ! defined $remote ) );
	return -1 if( ! ( $name =~ /^[a-z0-9\_\-\.]+$/ ) );
	return -1 if( Vhffs::Functions::valid_mail( $remote ) != 1 );

	return -2 if( ( defined ( $self->{'forward'}{$name} ) ) || ( defined( $self->{'boxes'}{$name} ) ) );

	$self->{'forward'}{$name}{'local_part'} = $name;
	$self->{'forward'}{$name}{'domain'} = $self->{'domain'};
	$self->{'forward'}{$name}{'remote_name'} = $remote;
    #self->{'forward'}{$name}{'changed'} = 1;
	
	return 1;
}


sub addbox
{
	my $self = shift;
	my $name = shift;
	my $password = shift;

	return -1 if( ( ! defined $password ) || ( ! defined $name ) );
	return -1 if( ! ( $name =~ /^[a-z0-9\_\-]+$/ ) );
	return -1 if( ! ( $password =~ /^[a-zA-Z0-9\_\-]+$/ ) );
	return -2 if( ( defined ( $self->{'boxes'}{$name} ) ) || ( defined( $self->{'forward'}{$name} ) ) );

	$self->{'boxes'}{$name}{'local_part'} = $name;
	$self->{'boxes'}{$name}{'password'} = crypt_pwd( $password );
	$self->{'boxes'}{$name}{'domain'} = $self->{'domain'};
	
	return 1;
}

sub delbox
{
	my $self = shift;
	my $name = shift;

	return -1 if( ! defined $name );
	return -1 if( ! ( $name =~ /^[a-zA-Z0-9\_\-]+$/ ) );

	undef $self->{boxes}{$name};

	my $query = "DELETE FROM vhffs_boxes WHERE local_part='".$name."' AND domain='".$self->{'domain'}."'";
	my $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );

	$request->execute;
}


sub delforward
{
	my $self = shift;
	my $name = shift;

	return -1 if( ! defined $name );

	undef $self->{forward}{$name};
	return -1 if( ! ( $name =~ /^[a-zA-Z0-9\_\-]+$/ ) );

	my $query = "DELETE FROM vhffs_forward WHERE local_part='".$name."' AND domain='".$self->{'domain'}."'";
	my $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
	$request->execute;
}


# Returns a hashref with all forwards
# Ths key of this hash is the local part for the forward
sub get_forwards
{
	my $self = shift;
	return $self->{'forward'};
}

sub exists
{
	my $self = shift;
	my $name = shift;

	return 1 if( ( $self->exists_forward( $name ) ==1 ) || ( $self->exists_box( $name ) ==1 ) );
	return 0;
}

sub exists_forward
{
	my $self = shift;
	my $name = shift;
	return -1 if( ! ( $name =~ /^[a-zA-Z0-9\_\-]+$/ ) );
    my $request = $self->{'db'}->{'DB_READ'}->prepare("SELECT * FROM vhffs_forward where domain='".$self->{'domain'}."' AND local_part='".$name."'") or return -1;
    return 1 if( $request->execute() > 0 );
	return 0;
}

sub exists_box
{
	my $self = shift;
	my $name = shift;
	return -1 if( ! ( $name =~ /^[a-zA-Z0-9\_\-]+$/ ) );
    my $request = $self->{'db'}->{'DB_READ'}->prepare("SELECT * FROM vhffs_boxes where domain='".$self->{'domain'}."' AND local_part='".$name."'") or return -1;
    return 1 if( $request->execute() > 0 );
	return 0;
}




#Following functions is needed by courier to fetch accounts

sub crypt_pwd
{
	my $clear = shift;
	my $salt  = join '', ('.', '/', 0..9,'A'..'Z', 'a'..'z')[rand 64, rand 64];
	return crypt( $clear , $salt );
}

# Returns an hashref with all bosex
# The key of this hash is the local_part of each mailbox
sub get_boxes
{
	my $self = shift;
	return $self->{'boxes'};
}

sub get_title
{
	my $self = shift;
	return $self->{'domain'};
}

sub get_domain
{
	my $self = shift;
	return $self->{'domain'};
}



###########################
# ospam function only use when the mail domain
# if fetched
# It can explain if a box ue antispam or not
# for example $mail->use_nospam( 'myaccount' );
# returns 	-1 if error
# 			0 if nospam is used
#			1 if spam is used
sub use_nospam
{
	my $self = shift;
	my $box = shift;

	return -1 if(  ! defined( $self->{'boxes'}->{$box} ) );
	return 1 if( $self->{'boxes'}->{$box}{'nospam'} == 1 );
	return 0;
}

sub use_novirus
{
	my $self = shift;
	my $box = shift;

	return -1 if(  ! defined( $self->{'boxes'}->{$box} ) );
	return 1 if( $self->{'boxes'}->{$box}{'novirus'} == 1 );
	return 0;
}



sub get_boxespath
{
	my $self = shift;
	return $self->{'boxes_path'};
}

sub get_catchall
{
	my $self = shift;
	
	if( defined $self->{'catchall'} )
	{
		return $self->{'catchall'};
	}
	else
	{
		return "";
	}
}

sub get_unixuser
{
	my( $self , $value ) = @_;
	$self->{'unix_user'} = $value;
}

sub set_catchall
{
	my( $self , $value ) = @_;
	use Vhffs::Functions;
	if( $value eq "" || Vhffs::Functions::valid_mail( $value ) == 1 )
	{
		$self->{'catchall'} = $value;
		return 1;
	}
	return -1;
}

sub set_user
{
	my( $self , $user ) = @_;
	$self->{'user'} = $user;
}

sub set_group
{
	my( $self , $group ) = @_;
	$self->{'group'} = $group;
}

sub get_owneruid
{
	my $self = shift;
	return $self->{'owner_uid'};
}

sub get_ownergid
{
	my $self = shift;
	return $self->{'owner_gid'};
}


sub getall
{
    my $vhffs = shift;
    my $state = shift;
    my $name = shift;
    my $group = shift;
	
    my $query;
    my $request;

    my $objs;
    my $result;
    my $tmp;

    $query = "SELECT m.domain FROM  vhffs_mxdomain m, vhffs_object o WHERE m.object_id = o.object_id";
	$query .= " AND o.state='".$state."'" if( defined $state );
	$query .= " AND m.domain LIKE '%".$name."%'" if( defined $name );
	$query .= " AND m.owner_gid='".$group->get_gid."'" if( defined $group );
    $request = $vhffs->{'db'}->{'DB_READ'}->prepare( $query );
    my $rows = $request->execute;

    return undef if( $rows <= 0);

    while( $result = $request->fetchrow_hashref() )
    {
    	$tmp = new Vhffs::Services::Mail( $vhffs , $result->{'domain'} );
    	push( @{$objs} , $tmp ) if( ( defined ( $tmp ) ) && ( $tmp->fetch > 0 ) )
    }
    return $objs;
}



1;
