#!/usr/bin/perl
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


# This file is a part of VHFFS4 software, a hosting platform suite
# Please respect the licence of this file and whole program

# Author : soda < god at gunnm dot org >

package Vhffs::Services::Mailing;

use base qw(Vhffs::Object);
use strict;
use DBI;


# Create a new instance of the current class
sub new
{
    my $main;
    my $domain;
    my $this;
    my $self;
    my $class;
    my $user;
    my $group;
    my $local;

    #( $class , $main , $domain , $user , $group ) = @_;
    
    $class = shift;
    $main = shift;
    $local = shift;
    $domain = shift;
    $user = shift;
    $group = shift;

    $this = {};

    #If the domain is not specified by argument, return an undef instance
    return undef if ( ! defined $domain );
    return undef if ( ! defined $local );

    #return undef if( $user->fetch < 0);
    $this = $class->SUPER::new( $main , 1 , '401' );
    

    return undef if( ! ( $local =~ /^[a-zA-Z0-9\-\.]+$/ ) );
    return undef if( ! ( $domain =~ /^[a-zA-Z0-9\-\.]+$/ ) );
    $this->{'domain'} = $domain if ( defined $domain );
    $this->{'local_part'} = $local if ( defined $local );


    $this->{'user'} = $user if( defined $user );
    $this->{'group'} = $group if( defined $group);

    bless( $this , $class );

    return $this;
}


# Create the object in the database
sub create
{
    my $self = shift;
    my $query;
    my $request;
    my $rows;

    $self->{'admin'} = "nobody\@nobody.com" if( ! defined $self->{admin} );

    return -1 if( ! defined $self->{'user'} );
    return -1 if( ! defined $self->{'group'} );

    # If user or group^does not exists in the VHFFS database, we return -1, the object is not created
    return -1 if( $self->{'user'}->fetch < 0 );
    return -1 if( $self->{'group'}->fetch < 0 );

    $query = "SELECT * FROM vhffs_ml where domain='".$self->{'domain'}."' AND local_part='".$self->{'local_part'}."'";
    $request = $self->{'db'}->{'DB_READ'}->prepare( $query ) or return -1;
    $rows = $request->execute() or return -3;
    return -1 if( $rows >= 1 );
    
    $self->{'owner_uid'} = $self->{'user'}->get_uid;
    $self->{'owner_gid'} = $self->{'group'}->get_gid;
    
    if( $self->{'domain'} ne $self->{'main'}->get_config->get_service("mailing")->{'default_domain'} )
    {
	$query = "SELECT * FROM vhffs_mxdomain WHERE domain='".$self->{domain}."' AND owner_gid='".$self->{'group'}->get_gid."'";
	$request = $self->{'db'}->{'DB_READ'}->prepare( $query );
	$rows = $request->execute;
	return -4 if ( $rows != 1 );
    }

    my $oid = $self->SUPER::create();
    return -3 if ( $oid lt 0 );

    #Get the max mx_id and compute the new mx_id
    my $ml_id = 1;
    my $resultuid = $self->{'db'}->{'DB_READ'}->selectall_arrayref("SELECT MAX(ml_id) FROM vhffs_ml");
    $ml_id = $resultuid->[0][0] if ( defined $resultuid->[0][0] );
    $ml_id++;
    
    #Finally, create the object in the database
    $query = "INSERT INTO vhffs_ml VALUES( '".$ml_id."' , '".$self->{'local_part'} . "', '".$self->{'domain'}."' , '".$self->{'local_part'} . "' , '".$self->{'user'}->get_uid."' , '".$self->{'group'}->get_gid."', '$oid'  )";
    $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
    $request->execute or return -2;


    $query = "INSERT INTO vhffs_ml_prefs VALUES( '".$ml_id."' , '".$self->{'admin'} . "' ,  '0' , '0' , '1' ,  '1' , '0' , ''  )";
    $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
    $request->execute or return -2;

    return $ml_id;
}


# Get all data about this mail object in the databse
# Informations about the mail domain is stored in $ref->
# Inforations about forwards are stored in $ref->{'forward'}
# Informations about mailboxes are stored in $ref->{'boxes'}
sub fetch
{
    my $self = shift;

    my ($request , $result , $request2 , $result2 , $query );
    my $rows;
    $query = "SELECT * FROM vhffs_ml where domain='".$self->{'domain'}."' AND local_part='".$self->{'local_part'}."'";
    $request = $self->{'db'}->{'DB_READ'}->prepare( $query ) or return -1;
    $rows = $request->execute() or return -3;
    return -1 if( $rows == 0 );
    $result = $request->fetchrow_hashref();

    foreach ( keys %{$result} )
    {
	$self->{$_} = $result->{$_}
    }

    #Fetch the object part( SUPER class)
    return -2 if ($self->SUPER::fetch < 0 ); 


    $query = "SELECT * FROM vhffs_ml_prefs where ml_id='".$self->{'ml_id'}."'";
    $request = $self->{'db'}->{'DB_READ'}->prepare( $query ) or return -1;
    $request->execute() or return -3;
    $result = $request->fetchrow_hashref();

    foreach ( keys %{$result} )
    {
	$self->{$_} = $result->{$_}
    }


    $query = "SELECT * FROM vhffs_ml_subscribers where ml_id='".$self->{'ml_id'}."'";
    $request = $self->{'db'}->{'DB_READ'}->prepare( $query ) or return -1;
    $request->execute() or return -3;
    $result = $request->fetchall_hashref('member');
    my $v;
    foreach ( keys %{$result} )
    {
	foreach $v ( keys %{$result->{$_}} )
	{
	    $self->{subs}{$_}{$v} = $result->{$_}{$v};
	}
    }

    return 1;	
}


# Commit all changes of the current instance in the database
sub commit
{
    my $self = shift;	

    my( $request , $result , $rows , $query , $name , $password);

#	$self->{'catchall'} = "" if( ! defined $self->{'catchall'} );

    $query = "UPDATE vhffs_ml SET prefix='".$self->{'prefix'}."' WHERE ml_id='".$self->{'ml_id'}."'";
    $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
    $request->execute;

    $query = "UPDATE vhffs_ml_prefs SET admin='".$self->{'admin'}."' , open_post='".$self->{'open_post'}."', open_archive='".$self->{'open_archive'}."' , reply_to='".$self->{'reply_to'}. "', moderated='".$self->{moderated}."' , open_sub='".$self->{'open_sub'}."'  WHERE ml_id='".$self->{'ml_id'}."'";
    $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
    $request->execute;


    return -3 if( $self->SUPER::commit < 0 );

    return 1;
}

sub change_right_for_sub
{
    my $self = shift;
    my $subscriber = shift;
    my $right = shift;

    return -1 if( Vhffs::Functions::valid_mail( $subscriber ) != 1 );
    return -1 if( ! ( $right =~ /^[0-9]+$/ ) );

    my $query = "UPDATE vhffs_ml_subscribers  SET perm='".$right."' WHERE ml_id='".$self->{'ml_id'}."' AND member='".$subscriber."'";
    my $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
    $request->execute or return -2;

    return 1;
}


sub add_sub
{
    my $self = shift;
    my $subscriber = shift;
    my $right = shift;

    return -1 if( Vhffs::Functions::valid_mail( $subscriber ) != 1 );
    return -2 if( $subscriber =~ /.*<.*/ );
    return -2 if( $subscriber =~ /.*>.*/ );
    return -2 if( $subscriber =~ /.*\s.*/ );
    return -1 if( ! ( $right =~ /^[\d]+$/ ) );

    my $query = "SELECT * FROM vhffs_ml_subscribers WHERE ml_id='".$self->{ml_id}."' AND member='".$subscriber."'";
    my $request = $self->{'db'}->{'DB_READ'}->prepare( $query );
    my $rows = $request->execute;
    return -1 if ( $rows != 0 );

    my $sid = 1;
    $query = "SELECT MAX(sub_id) FROM vhffs_ml_subscribers";
    my $resultsid = $self->{'db'}->{'DB_READ'}->selectall_arrayref( $query );
    $sid = $resultsid->[0][0] if ( defined $resultsid->[0][0] );
    $sid++;

    $query = "INSERT INTO vhffs_ml_subscribers VALUES( '".$sid."' , '".$subscriber."' , '".$right."' ,  '1' , 'nohash' , '".$self->{'ml_id'}."')";
    $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
    $request->execute or return -2;

    return 1;
}



#add a subscriber, return undef if already exists
sub add_sub_with_reply
{
    use Digest::MD5;
    my $self = shift;
    my $subscriber = shift;
    my $hash;

    return undef if( Vhffs::Functions::valid_mail( $subscriber ) != 1 );

    #check if sub already exists
    my $query = "SELECT * FROM vhffs_ml_subscribers WHERE ml_id='".$self->{ml_id}."' AND member='".$subscriber."'";
    my $request = $self->{'db'}->{'DB_READ'}->prepare( $query );
    my $rows = $request->execute;
    return undef if ( $rows != 0 );

    my $sid = 1;
    $query = "SELECT MAX(sub_id) FROM vhffs_ml_subscribers";
    my $resultsid = $self->{'db'}->{'DB_READ'}->selectall_arrayref( $query );
    $sid = $resultsid->[0][0] if ( defined $resultsid->[0][0] );
    $sid++;


    $hash = Digest::MD5::md5_hex( Vhffs::Functions::generate_random_password );
    
    $query = "INSERT INTO vhffs_ml_subscribers VALUES( '".$sid."' , '".$subscriber."' , '".Vhffs::Constants::ML_RIGHT_SUB_WAITING_FOR_REPLY."' ,  '1' , '".$hash."' , '".$self->{'ml_id'}."')";
    $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
    $request->execute or return -2;

    return $hash;
}



sub del_sub
{
    my $self = shift;
    my $subscriber = shift;

    return -1 if( Vhffs::Functions::valid_mail( $subscriber ) != 1 );


    return -2 if ( $self->sub_exists( $subscriber )  !=  1);


    my $query = "DELETE FROM vhffs_ml_subscribers  WHERE ml_id='".$self->{'ml_id'}."' AND member='".$subscriber."'";
    my $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
    $request->execute or return -1;

    return 1;
}

sub change_state_for_sub
{
    my $self = shift;
    my $subscriber = shift;
    my $right = shift;

    return -1 if( Vhffs::Functions::valid_mail( $subscriber ) != 1 );

    return -2 if ( $self->sub_exists( $subscriber ) != 1);

    my $query = "UPDATE vhffs_ml_subscribers SET perm='".$right."' WHERE ml_id='".$self->{'ml_id'}."' AND member='".$subscriber."'";
    my $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
    $request->execute or return -1;

    return 1;
}

sub clear_hash
{
    my $self = shift;
    my $subscriber = shift;

    return -1 if( Vhffs::Functions::valid_mail( $subscriber ) != 1 );

    return -2 if ( $self->sub_exists( $subscriber ) != 1);

    my $query = "UPDATE vhffs_ml_subscribers SET hash=' ' WHERE ml_id='".$self->{'ml_id'}."' AND member='".$subscriber."'";
    my $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
    $request->execute or return -1;

    return 1;
}




sub del_sub_with_reply
{
    use Digest::MD5;

    my $self = shift;
    my $subscriber = shift;

    return -1 if( Vhffs::Functions::valid_mail( $subscriber ) != 1 );


    return -2 if ( $self->sub_exists( $subscriber ) != 1);

    return -3 if( ( $self->get_members->{$subscriber}{perm} != Vhffs::Constants::ML_RIGHT_SUB ) && ( $self->get_members->{$subscriber}{perm} != Vhffs::Constants::ML_RIGHT_ADMIN ) );
    
    my $hash = Digest::MD5::md5_hex( Vhffs::Functions::generate_random_password );
    my $query = "UPDATE vhffs_ml_subscribers SET perm='".Vhffs::Constants::ML_RIGHT_SUB_WAITING_FOR_DEL."', hash='".$hash."' WHERE ml_id='".$self->{'ml_id'}."' AND member='".$subscriber."'";
    my $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
    $request->execute or return -1;

    return $hash;
}

sub sub_exists
{
    my $self = shift;
    my $subscriber = shift;

    return -1 if( Vhffs::Functions::valid_mail( $subscriber ) != 1 );

    my $query = "SELECT * FROM vhffs_ml_subscribers WHERE ml_id='".$self->{ml_id}."' AND member='".$subscriber."'";
    my $request = $self->{'db'}->{'DB_READ'}->prepare( $query );
    my $rows = $request->execute;
    return -1 if ( $rows == 0 );
    return 1;
}


sub delete
{
    my $self = shift;
    my $query;
    my $request;

    $query = "DELETE FROM vhffs_ml_subscribers  WHERE ml_id='".$self->{'ml_id'}."'";
    $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
    $request->execute or return -1;

    $query = "DELETE FROM vhffs_ml_prefs  WHERE ml_id='".$self->{'ml_id'}."'";
    $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
    $request->execute or return -1;


    $query = "DELETE FROM vhffs_ml WHERE ml_id='".$self->{'ml_id'}."'";
    $request = $self->{'db'}->{'DB_WRITE'}->prepare( $query );
    $request->execute or return -1;

    $self->SUPER::delete;

    return 1;
}


sub get_lang_for_sub
{
    my $vhffs = shift;
    my $sub = shift;
    my $query;
    my $request;


    $query = "SELECT lang FROM vhffs_ml_lang WHERE member='".$sub."'";
    $request = $vhffs->{'db'}->{'DB_READ'}->prepare( $query );
    return undef if ( $request->execute <= 0 );
    
    my $result = $request->fetchrow_arrayref;
    return $result->[0];
}


sub set_lang_for_sub
{
    my $vhffs = shift;
    my $sub = shift;
    my $lang = shift;
    my $query,
    my $request;

	if( $lang =~ /^fr$|^fr_FR$|^french$/ )
	{
		$lang = "fr_FR";
	}
	elsif( $lang =~ /^es$|^es_ES$|^spanish$/ )
	{
		$lang = "es_ES";
	}
	else
	{
		$lang = "en_US";
	}

    $query = "SELECT * FROM vhffs_ml_lang WHERE member='".$sub."'";
    $request = $vhffs->{'db'}->{'DB_READ'}->prepare( $query );
    if ( $request->execute <= 0 )
    {
	$query = "INSERT INTO vhffs_ml_lang VALUES( '".$sub."' , '".$lang."' )";
    }
    else
    {
	$query = "UPDATE vhffs_ml_lang SET lang='".$lang."' WHERE member='".$sub."'";
    }

    $request = $vhffs->{'db'}->{'DB_WRITE'}->prepare( $query );
    $request->execute or return -1;

}

sub get_localpart
{
    my $self = shift;
    return $self->{'local_part'};
}


sub get_open_archive
{
    my $self = shift;
    return $self->{'open_archive'};
}

sub get_open_post
{
    my $self = shift;
    return $self->{'open_post'};
}

sub get_open_sub
{
    my $self = shift;
    return $self->{'open_sub'};
}



sub get_replyto
{
    my $self = shift;
    return $self->{'reply_to'};
}


sub get_url
{
    my $self = shift;
    return $self->{'url'};
}


sub get_moderated
{
    my $self = shift;
    return $self->{'moderated'};
}

sub get_admin
{
    my $self = shift;
    return $self->{'admin'};
}


sub get_domain
{
    my $self = shift;
    return $self->{'domain'};
}


sub get_prefix
{
    my $self = shift;
    return $self->{'prefix'};
}

sub get_members
{
    my $self = shift;
    return $self->{subs};
}


sub set_replytolist
{
    my $self;
    my $string = $self->{'local_part'} ."\@" . $self->{'domain'} ;
    return $self->set_replyto( $string );
}

sub set_replyto
{
    my( $self , $value ) = @_;
	return -2 if( ( $value != 0 ) && ( $value != 1 ) );
    $self->{'reply_to'} = $value;
    return 1;
}


sub set_moderated
{
    my( $self , $value ) = @_;
    $self->{'moderated'} = $value;
}

sub set_open_post
{
    my( $self , $value ) = @_;
    $self->{'open_post'} = $value;
}


sub set_open_archive
{
    my( $self , $value ) = @_;
    $self->{'open_archive'} = $value;
}

sub set_open_sub
{
    my( $self , $value ) = @_;
    $self->{'open_sub'} = $value;
}

sub set_admin
{
    my $self = shift;
    my $value = shift;
    $self->{'admin'} = $value;
}

sub set_prefix
{
    my( $self , $value ) = @_;

    $self->{'prefix'} = quotemeta $value;
}

sub set_user
{
    my( $self , $user ) = @_;
    $self->{'user'} = $user;
}

sub set_group
{
    my( $self , $group ) = @_;
    $self->{'group'} = $group;
}

sub getall_subs
{
    my $self = shift;
    return ( keys %{$self->{subs}} );
}

sub getall
{
    my $vhffs = shift;
    my $state = shift;
    my $name = shift;
    my $group = shift;
    my $domain = shift;

    my $query;
    return undef if( ! defined $vhffs );

	$query = "SELECT * FROM vhffs_ml ml, vhffs_object o WHERE o.object_id=ml.object_id";
	$query .= " AND o.state='".$state."'" if( defined $state );
	$query .= " AND ml.owner_gid='".$group->get_gid."'" if( defined $group );
    $query .= " AND ( ( local_part LIKE '%".$name."%' ) OR (domain LIKE '%".$name."%' ) )" if( defined $name );
    $query .= " AND ( domain='".$domain."')" if( defined $domain );

    my $request = $vhffs->get_db_object->{'DB_READ'}->prepare( $query );
    my $rows = $request->execute() or return -3;
	return undef if( $rows <= 0 );
    my $result = $request->fetchall_hashref('ml_id');
    my $object;
    my @plop;


    
    foreach( keys %{$result} ) 
    {
	$object = new Vhffs::Services::Mailing( $vhffs , $result->{$_}{local_part} , $result->{$_}{domain} );
	push( @plop , $object ) if( ( defined $object ) && ( $object->fetch > 0 ) );
    }

    return( \@plop );
}



#Returns all the lists for a group
sub getall_by_group
{
    my $vhffs = shift;
    my $group = shift;

    my $query;
    return undef if( ! defined $vhffs );

	$query = "SELECT * FROM vhffs_ml ml, vhffs_object o WHERE o.object_id=ml.object_id";
	$query .= " AND ml.owner_gid='".$group->get_gid."'" if( defined $group );

    my $request = $vhffs->get_db_object->{'DB_READ'}->prepare( $query );
    my $rows = $request->execute() or return -3;
	return undef if( $rows <= 0 );
    my $result = $request->fetchall_hashref('ml_id');
    my $object;
    my @plop;


    
    foreach( keys %{$result} ) 
    {
		$object = new Vhffs::Services::Mailing( $vhffs , $result->{$_}{local_part} , $result->{$_}{domain} );
		push( @plop , $object ) if( ( defined $object ) && ( $object->fetch > 0 ) );
    }

    return( \@plop );
}



sub get_title
{
    my $self = shift;
    return $self->{local_part} . "\@" . $self->{domain};
}

sub get_ownergid
{
    my $self = shift;
    return $self->{'owner_gid'};
}

sub get_owneruid
{
    my $self = shift;
    return $self->{'owner_uid'};
}

sub get_listname
{
    my $self = shift;
    return $self->get_localpart."\@".$self->get_domain;
}

1;
