#!/usr/bin/perl -w 
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


use POSIX qw(locale_h);
use HTML::Template;
use locale;
use Locale::gettext;
use CGI;
use CGI::Session;
use strict;


use Vhffs::User;
use Vhffs::Group;
use Vhffs::Main;
use Vhffs::Panel::Main;
use Vhffs::Panel::Menu;
use Vhffs::Stats;
use Vhffs::Constants;
my $panel = new Vhffs::Panel::Main();
if(!$panel)  {
        exit 0;
}

my $vhffs = $panel->{'vhffs'};
my $session = $panel->{'session'};
my $maintemplate = $panel->{'template'};
my $user = $panel->{'user'};
#my $group = $panel->{'group'};
my $projectname = $session->param("project");
my $cgi = $panel->{'cgi'};
my $name = $cgi->param("NAME");
my $template;

my $templatedir = $vhffs->get_config->get_templatedir;

if( ($user->is_moderator != 1 ) && ( $user->is_admin != 1 ) )
{
	
	$template = new HTML::Template( filename => $templatedir."/misc/simplemsg.tmpl" );
	my $message = gettext( "You are not allowed to see it");
	$template->param( MESSAGE => $message );
}
else
{
	$template = new HTML::Template( filename => $templatedir."/admin/misc/list.tmpl" );

	if( defined( $name ) )
	{
		$template->param( TITLE => gettext("Search result for domain") . ": " .$name );
	}
	else
	{
		$template->param( TITLE => gettext("List of all domains") );
	}

	use Vhffs::Panel::Web;

	my $domains = Vhffs::Panel::DNS::getall_domains( $vhffs , $name );

	if( defined $domains )
	{
		my $output = "";
		foreach( sort ( keys %{$domains} ) )
		{
			my $subtemplate;
			if( $user->is_admin == 1 )  {
				$subtemplate = new HTML::Template( filename => $templatedir."/admin/dns/part.tmpl" );
			}  else  {
				$subtemplate = new HTML::Template( filename => $templatedir."/admin/dns/part-modo.tmpl" );
			}
			$subtemplate->param(  DOMAIN => $domains->{$_}{'domain'} ) if( defined $domains->{$_}{'domain'} );
			$subtemplate->param( GROUP => Vhffs::Group::get_name_by_gid( $vhffs , $domains->{$_}{'owner_gid'} ) ) if( defined $domains->{$_}{'owner_gid'} );

			if( $domains->{$_}{'state'} == Vhffs::Constants::WAITING_FOR_CREATION )
			{
				$subtemplate->param( STATE => gettext("Waiting for creation") );
			}
			elsif( $domains->{$_}{'state'} == Vhffs::Constants::ACTIVATED )
			{
				$subtemplate->param( STATE => gettext("Activated") );
			}
			elsif( $domains->{$_}{'state'} == Vhffs::Constants::TO_DELETE )
			{
				$subtemplate->param( STATE => gettext("Waiting for delete") );
			}
			elsif( $domains->{$_}{'state'} == Vhffs::Constants::WAITING_FOR_VALIDATION )
			{
				$subtemplate->param( STATE => gettext("Waiting for validation") );
			}
			
			if( $user->is_admin == 1 )  {
				$subtemplate->param( OID => $domains->{$_}{'object_id'} ) if( defined $domains->{$_}{'object_id'} );
				$subtemplate->param( SEND => gettext("Modify this domain") );
			}

			$output .= $subtemplate->output;
		}
		$template->param( LIST => $output );
	}


}


display Vhffs::Panel::Main($panel, $template->output);
