#!/usr/bin/perl -w 
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


use POSIX qw(locale_h);
use HTML::Template;
use locale;
use Locale::gettext;
use CGI;
use CGI::Session;
use strict;


use Vhffs::User;
use Vhffs::Group;
use Vhffs::Main;
use Vhffs::Panel::Main;
use Vhffs::Panel::Menu;
use Vhffs::Stats;
use Vhffs::Constants;
my $panel = new Vhffs::Panel::Main();
if(!$panel)  {
        exit 0;
}

my $vhffs = $panel->{'vhffs'};
my $session = $panel->{'session'};
my $maintemplate = $panel->{'template'};
my $user = $panel->{'user'};
#my $group = $panel->{'group'};
my $projectname = $session->param("project");
my $cgi = $panel->{'cgi'};
my $name = $cgi->param("NAME");
my $template;
my $templatesdir = $vhffs->get_config->get_templatedir;


if( $user->is_admin != 1 )
{
	$template = new HTML::Template( filename => "$templatesdir/misc/simplemsg.tmpl" );
	my $message = gettext( "You are not allowed to see it");
	$template->param( MESSAGE => $message );
}
else
{
	$template = new HTML::Template( filename => "$templatesdir/admin/misc/list.tmpl" );

	if( defined( $name ) )
	{
		$template->param( TITLE => gettext("Search result "));
	}
	else
	{
		$template->param( TITLE => gettext("All objects List") );
	}

	use Vhffs::Object;

	my $objs = Vhffs::Object::getall( $vhffs , $name );

	if( defined $objs )
	{
		my $output = "";
		foreach( sort {$a - $b} ( keys %{$objs} ) )
		{
			my $subtemplate;
			$subtemplate = new HTML::Template( filename => "$templatesdir/admin/object/part.tmpl" );

			$subtemplate->param( USER => Vhffs::User::get_name_by_uid( $vhffs , $objs->{$_}{'owner_uid'} ) ) if( defined $objs->{$_}{'owner_uid'} );
			$subtemplate->param( HISTORY => gettext("History") );
			$subtemplate->param( OID => $objs->{$_}{'object_id'} ) if( defined $objs->{$_}{'object_id'} );

			if( $objs->{$_}{'state'} == Vhffs::Constants::WAITING_FOR_CREATION )
			{
				$subtemplate->param( STATE => gettext("Waiting for creation") );
			}
			elsif( $objs->{$_}{'state'} == Vhffs::Constants::ACTIVATED )
			{
				$subtemplate->param( STATE => gettext("Activated") );
			}
			elsif( $objs->{$_}{'state'} == Vhffs::Constants::TO_DELETE )
			{
				$subtemplate->param( STATE => gettext("Waiting for delete") );
			}
			elsif( $objs->{$_}{'state'} == Vhffs::Constants::WAITING_FOR_VALIDATION )
			{
				$subtemplate->param( STATE => gettext("Waiting for validation") );
			}
			
			$subtemplate->param( SEND => gettext("Edit this object") );

			$output .= $subtemplate->output;
		}
		$template->param( LIST => $output );
	}
	

}

display Vhffs::Panel::Main($panel, $template->output);
