#!/usr/bin/perl -w 
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


use POSIX qw(locale_h);
use HTML::Template;
use locale;
use Locale::gettext;
use CGI;
use CGI::Session;
use strict;


use Vhffs::User;
use Vhffs::Group;
use Vhffs::Main;
use Vhffs::Panel::Main;
use Vhffs::Panel::Menu;
use Vhffs::Stats;
use Vhffs::Constants;
my $panel = new Vhffs::Panel::Main();
if(!$panel)  {
        exit 0;
}

my $vhffs = $panel->{'vhffs'};
my $session = $panel->{'session'};
my $maintemplate = $panel->{'template'};
my $user = $panel->{'user'};
#my $group = $panel->{'group'};
my $projectname = $session->param("project");
my $cgi = $panel->{'cgi'};
my $name = $cgi->param("NAME");
my $template;
my $templatesdir = $vhffs->get_config->get_templatedir;


if( ($user->is_moderator != 1 ) && ( $user->is_admin != 1 ) )
{
	
	$template = new HTML::Template( filename => "$templatesdir/misc/simplemsg.tmpl" );
	my $message = gettext( "You are not allowed to see it");
	$template->param( MESSAGE => $message );
}
else
{
	$template = new HTML::Template( filename => "$templatesdir/admin/misc/list.tmpl" );

	if( defined( $name ) )
	{
		$template->param( TITLE => gettext("Search result for Download repository") . ": " .$name );
	}
	else
	{
		$template->param( TITLE => gettext("List of all Download repositories") );
	}

	use Vhffs::Services::Repository;

	my $repos = Vhffs::Services::Repository::getall( $vhffs , undef , $name , undef );
	my $repo;
	my $output = "";

	if( defined $repos )
	{
		foreach $repo ( @{$repos} )
		{
			my $subtemplate;
			if( $user->is_admin == 1 )  {
				$subtemplate = new HTML::Template( filename => "$templatesdir/admin/repository/part.tmpl" );
			}  else  {
				$subtemplate = new HTML::Template( filename => "$templatesdir/admin/repository/part-modo.tmpl" );
			}

			$subtemplate->param( NAME => $repo->get_name );
			$subtemplate->param( GROUP => $repo->get_group->get_groupname );
			$subtemplate->param( OID => $repo->get_oid );
			$subtemplate->param( STATE => gettext( Vhffs::Functions::status_string_from_status_id( $repo->get_status ) ) );
			
			if( $user->is_admin == 1 )  {
				$subtemplate->param( SEND => gettext("Modify this Download repository") );
			}  else  {
				$subtemplate->param( SEND => gettext("Show this Download repository") );
			}

			$output .= $subtemplate->output;
		}
		$template->param( LIST => $output );
	}
	

}

display Vhffs::Panel::Main($panel, $template->output);
