#!/usr/bin/perl -w
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


#Add a A to a domain
#This is the CGI part, it uses the main API of VHFFS


use POSIX qw(locale_h);
use HTML::Template;
use locale;
use Locale::gettext;
use CGI;
use CGI::Session;
use strict;


use Vhffs::User;
use Vhffs::Group;
use Vhffs::Main;
use Vhffs::Panel::Main;
use Vhffs::Panel::Menu;

my $panel = new Vhffs::Panel::Main();
if(!$panel)  {
        exit 0;
}


#Fetch some vars from the main API
my $cgi = $panel->{'cgi'};
my $vhffs = $panel->{'vhffs'};
my $domain_name = $cgi->param("DOMAIN");
my $name = $cgi->param("PREFIX");
my $ip = $cgi->param("IP");
my $user = $panel->{'user'};
my $question = $cgi->param("QUESTION");
my $dns = new Vhffs::Services::DNS( $vhffs , $domain_name , $user );
my $session = $panel->{'session'};
my $maintemplate = $panel->{'template'};
my $group = $panel->{'group'};
my $projectname = $session->param("project");

my $templatedir = $vhffs->get_config->get_templatedir;

my $template;
my $output = "";
my $message;

$template = new HTML::Template( filename => $templatedir."/misc/simplemsg.tmpl" );

if( ( ! defined $domain_name ) || ( ! defined $dns ) || ( ! defined $name ) || ( ! defined $ip ))
{
	$message = gettext( "CGI Error!");
}
elsif( $dns->fetch < 0 )
{
	$message = gettext( "Cannot get informations on this object");
}
elsif( ( Vhffs::Acl::what_perm_for_user( $user , $dns , $vhffs ) < Vhffs::Constants::ACL_MODIFY ) && ( $user->is_admin != 1 ) )
{
    $message = gettext("You're not allowed to do this (ACL rights)");
}
elsif( $dns->get_status != Vhffs::Constants::ACTIVATED )
{
    $message = gettext("You're not allowed to do this (ACL rights)");
}
else
{
    #If $question equals to 1, we must use the adress defined in the VHFFS config file
    #With this option, the user just put the name for the A redir and the API takes the IP from the config file
    if( ( defined $question ) && ( $question == 1 ) )
    {
	# add_a( $foo , undef ) will add to the domain
	#                                            foo.domain  IN    A    adress_in_config_file
	if( $dns->add_a( $name , undef ) < 0 )
	{
	    $message = gettext("Cannot add this ressource to this domain");
	}
	else
	{
	    $message = gettext("Resource successfully added to this domain");
	}
    }
    else
    {
	#Check if the IP is valid and add it on the domain
	if( ! ( $ip =~ /[\d+\.].[\d+\.].[\d+\.].[\d+]/ ) )
	{
	    $message = gettext("Invalid IP");
	}
	elsif( $dns->add_a( $name , $ip ) < 0 )
	{
	    $message = gettext("Cannot add this ressource to this domain");
	}
	else
	{
	    $message = gettext("Resource successfully added to this domain");
	}
    }
}

$template->param( MESSAGE => $message );
set_refresh_url Vhffs::Panel::Main($panel, "/dns/prefs.pl?name=$domain_name", 0);
display Vhffs::Panel::Main($panel, $template->output);
