#!/usr/bin/perl -w
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


use POSIX qw(locale_h);
use HTML::Template;
use locale;
use Locale::gettext;
use CGI;
use CGI::Session;
use strict;

use Vhffs::User;
use Vhffs::Group;
use Vhffs::Main;
use Vhffs::Panel::Main;
use Vhffs::Panel::Menu;
use Vhffs::Panel::Group;

my $panel = new Vhffs::Panel::Main;
if(!$panel)  {
        exit 0;
}

my $vhffs = $panel->{'vhffs'};
my $session = $panel->{'session'};
my $maintemplate = $panel->{'template'};
my $user = $panel->{'user'};
my $group = $panel->{'group'};
my $groupname = $group->get_groupname;
my $cgi = $panel->{'cgi'};
my $message;

my $dbname = $groupname ."_". $cgi->param("DB_NAME");
my $dbuser = $dbname;
my $dbpass = $cgi->param("DB_PASS");

my $description = $cgi->param("DESCRIPTION");

my $templatedir = $vhffs->get_config->get_templatedir;

if( ( ! defined $description ) || ( ! defined $dbpass ) || ( ! defined $dbuser ) )
{
	$message = gettext("CGI Error!");
}
elsif( length( $dbpass ) < 3 )
{
	$message = gettext("Database password must contains at least 3 characters");
}
elsif( ! ( $dbpass =~ /^[a-zA-Z0-9]+$/ ) )
{
	$message = gettext("Password must contains only alphanum caracters");
}
elsif( ( length( $dbname ) < 3 ) || ( length( $dbname ) >= 16 ) )
{
	$message = gettext( "Database name must contain between 3 and 16 characters" );
}
else
{

	my $postgres = Vhffs::Panel::Pgsql::create_pgsql( $vhffs , $dbname , $user , $group , $dbuser , $dbpass );


	if( defined $postgres )
	{
		$postgres->set_description( $description );
		$postgres->commit;
		$message = gettext("The PostgreSQL object was successfully created !");
	}
	else
	{
		$message = sprintf( gettext("An error occured while creating the object %s %s"), $dbname, $dbuser );
	}
}

my $template = new HTML::Template( filename => $templatedir."/misc/simplemsg.tmpl" );
$template->param( MESSAGE => $message );

set_refresh_url Vhffs::Panel::Main($panel, "/panel.pl?project=$groupname", 0);
display Vhffs::Panel::Main($panel, $template->output);
