#!/usr/bin/perl
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


use Vhffs::Robots::Group;
use Vhffs::Main;
use Vhffs::Group;
use Vhffs::Constants;
use Vhffs::Services::Cvs;
use Vhffs::Services::DNS;
use Vhffs::Services::Httpd;
use Vhffs::Services::Mailing;
use Vhffs::Services::Mail;
use Vhffs::Services::Mysql;
use Vhffs::Services::Postgres;
use Vhffs::Services::Svn;
use Vhffs::Robots;


my $vhffs = init Vhffs::Main;

Vhffs::Robots::lock( $vhffs );

my $groups = Vhffs::Group::getall( $vhffs , Vhffs::Constants::TO_DELETE );
my $group;
my $objects;
my $object;
my $ok;

foreach $group ( @{$groups} )
{
	Vhffs::Robots::vhffs_log( sprintf(  "Will try to delete group %s",  $group->get_groupname), $vhffs);
	$ok = 1;

    $objects = Vhffs::Services::Cvs::getall( $vhffs , undef , undef , $group );
	$ok = 0 if( defined $objects );
    foreach $object ( @{$objects} )
    {
		Vhffs::Robots::vhffs_log( sprintf( "Remain cvs for group %s before delete" , $group->get_groupname ), $vhffs);
        $object->set_status( Vhffs::Constants::TO_DELETE );
		$object->commit;
    }

    $objects = Vhffs::Services::Svn::getall( $vhffs , undef , undef , $group );
	$ok = 0 if( defined $objects );
    foreach $object ( @{$objects} )
    {
		Vhffs::Robots::vhffs_log( sprintf( "Remain svn for group %s before delete" , $group->get_groupname ), $vhffs);
        $object->set_status( Vhffs::Constants::TO_DELETE );
		$object->commit;
    }

    $objects = Vhffs::Services::DNS::getall( $vhffs , undef , undef , $group );
	$ok = 0 if( defined $objects );
    foreach $object ( @{$objects} )
    {
		Vhffs::Robots::vhffs_log( sprintf( "Remain dns for group %s before delete" , $group->get_groupname ), $vhffs);
        $object->set_status( Vhffs::Constants::TO_DELETE );
		$object->commit;
    }

    $objects = Vhffs::Services::Mysql::getall( $vhffs , undef , undef , $group );
	$ok = 0 if( defined $objects );
    foreach $object ( @{$objects} )
    {
		Vhffs::Robots::vhffs_log( sprintf( "Remain mysql for group %s before delete" , $group->get_groupname ), $vhffs);
        $object->set_status( Vhffs::Constants::TO_DELETE );
		$object->commit;
    }

    $objects = Vhffs::Services::Postgres::getall( $vhffs , undef , undef , $group );
	$ok = 0 if( defined $objects );
   foreach $object ( @{$objects} )
   {
		Vhffs::Robots::vhffs_log( sprintf( "Remain pgsql for group %s before delete" , $group->get_groupname ), $vhffs);
        $object->set_status( Vhffs::Constants::TO_DELETE );
		$object->commit;
   }


    $objects = Vhffs::Services::Httpd::getall( $vhffs , undef , undef , $group );
	$ok = 0 if( defined $objects );
   foreach $object ( @{$objects} )
   {
		Vhffs::Robots::vhffs_log( sprintf( "Remain website for group %s before delete" , $group->get_groupname ), $vhffs);
        $object->set_status( Vhffs::Constants::TO_DELETE );
		$object->commit;
   }
    
	$objects = Vhffs::Services::Mailing::getall( $vhffs , undef , undef , $group );
	$ok = 0 if( defined $objects );
   foreach $object ( @{$objects} )
   {
		Vhffs::Robots::vhffs_log( sprintf( "Remain list for group %s before delete" , $group->get_groupname ), $vhffs);
        $object->set_status( Vhffs::Constants::TO_DELETE );
		$object->commit;
   }


	$objects = Vhffs::Services::Mail::getall( $vhffs , undef , undef , $group );
	$ok = 0 if( defined $objects );
   foreach $object ( @{$objects} )
   {
		Vhffs::Robots::vhffs_log( sprintf( "Remain mail for group %s before delete" , $group->get_groupname ), $vhffs);
        $object->set_status( Vhffs::Constants::TO_DELETE );
		if( $object->commit < 0 )
		{
			Vhffs::Robots::vhffs_log( "Error in delete group" , $vhffs);
		}
   }


	if( $ok == 1 )
	{
		Vhffs::Robots::vhffs_log( sprintf( "Delete group %s" , $group->get_groupname ), $vhffs);
		Vhffs::Robots::Group::delete( $vhffs , $group );
	}
	else
	{
		Vhffs::Robots::vhffs_log( sprintf( "Cannot elete group %s, items remains" , $group->get_groupname ), $vhffs);
	}

}

Vhffs::Robots::unlock( $vhffs );

exit( 0 );

