#!/usr/bin/perl
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


# This is a part of VHFFS platform
# Please consider the entire platform
# See http://vhffs.org

# by sod



use Vhffs::Robots::Cvs;
use Vhffs::Main;
use Vhffs::User;
use Vhffs::Group;

Vhffs::Robots::lock;

my $main = init Vhffs::Main;
my $cvss = Vhffs::Services::Cvs::getall( $main  , Vhffs::Constants::WAITING_FOR_MODIFICATION, undef , undef );
my $cvs;
my $dir;

my $readers_file;
my $passwd_file;
my $username;
my $groupname;

foreach $cvs ( @{$cvss} )
{

    $dir = $cvs->get_dir . "/CVSROOT/" ;   
    $readers_file = $dir."/readers";
    $passwd_file = $dir."/passwd";


    #Get real groupname and username
    $groupname = Vhffs::Group::get_name_by_gid( $main , $cvs->get_ownergid );
    $username = Vhffs::User::get_name_by_uid( $main , $cvs->get_owneruid );

    # Change files if cvs is public
    if( $cvs->is_public == 1 )
    {
		Vhffs::Robots::vhffs_log( sprintf("CVS, put repository %s public" , $cvs->get_cvsroot), $main );
	open( FILE , ">$readers_file" ) or die( "cannot open $readers_file" );
	print FILE "anonymous";
	close( FILE );

	open( FILE , ">$passwd_file" ) or die( "cannot open $passwd_file" );
	print FILE "anonymous::".$username;
	close( FILE );

	#Finally, change files permission
	chown $cvs->get_owneruid , $cvs->get_ownergid , $passwd_file;
	chown $cvs->get_owneruid , $cvs->get_ownergid , $readers_file;
    }
	else
	{
		Vhffs::Robots::vhffs_log( sprintf("CVS, put repository %s private" , $cvs->get_cvsroot), $main );
		if( -f $readers_file )
		{
			unlink( $readers_file );
		}

		if( -f $passwd_file )
		{
			unlink( $passwd_file );
		}
	}
}

Vhffs::Robots::unlock;

exit 0;
